//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
// VI.h
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
// SYNOPSIS:
// The generic DC power (VI) instrument driver defines the APIs for all the VI physical instrument drivers.
// The stratigy is to have the generic driver provide the superset of all VI functions, and have the phyical 
// drivers ignor APIs that they cannot support.
// This module implements a set of VI properties that represent atomic parameters to a VI instrument,
// amd a set of APIs that breakdown a users function into property values.  The phyical driver then builds
// a hardware manipulation APIs from the properties.
// The scheme allows for a totally general mapping from user generic APIs to physical APIs (whatever they might be).
// 
// IMPLEMENTATION:
// Data classes:
// VI_PR is the set of VI properties which represent the values that define the state of any VI pin.
// VI parameter enums which work with the generic APIs.
// UTL_VI class contains the API for unison users.
// 
// 2007-11-15 kps
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#pragma once

#include "GlobalEnums.h"

namespace UTL
{
// moved to VI_PR /vobs/idrivers/include/user/VISupport.h

/** @file
@brief Defines the @ref VIAPIs "VI.*" API calls 
@details This is even more info about the VI API calls. */
/** @defgroup VIAPIs VI (Voltage/Current) Instruments
@{
@ingroup Drivers
@par VI APIs
        
These commands are used for all of the system VI (voltage/current source) instruments.

@anchor VIsupport        
@par Supported Platforms and VI Instruments

*/
    /** @defgroup VI_Enums VI API Support Enums
    @{
    @ingroup VIAPIs
    @par Driver Support Enums

*/


// VI parameter enums for user APIs.
    enum VI_CONFIG_MODE
    {
        VI_CONFIG_PULSED,            /**< Configuration is for a pulsed force current */
        VI_CONFIG_CONTINUOUS         /**< Configuration is for a continuous force current */
    };

    enum VI_CONNECT_MODE
    {
        VI_MODE_LOCAL,             /**< Local sense */
        VI_MODE_REMOTE,            /**< Remote sense */
        VI_MODE_REMOTE_HIZ,        /**< High-impedance remote sense */
        VI_MODE_VM,                /**< Voltmeter on the remote sense lines */
        VI_MODE_THROUGH            /**< Close both sense and force loop relays for HDVI only */
    };

    enum VI_GATE_STATE
    {
        VI_GATE_ON,                /**< Gate on */
        VI_GATE_OFF_LOZ,           /**< Gate off, low-impedance mode */
        VI_GATE_OFF_HIZ            /**< Gate off, high-impedance mode */
    };

    enum VI_MEASURE_TYPE
    {
        VI_MEASURE_I,              /**< Measure current. */
        VI_MEASURE_V,              /**< Measure voltage. */
        VI_MEASURE_VDIFF,          /**< Measure diffential voltage. */
        VI_MEASURE_V_I,            /**< Measure voltage and current. */
        VI_MEASURE_RAIL_MAX,       /**< Measure high rail voltage. */
        VI_MEASURE_RAIL_MIN,       /**< Measure high low voltage. */
        VI_MEASURE_MS_HS,          /**< Measure MS-HS difference. */
        VI_MEASURE_MS_LS,          /**< Measure MS-LS difference. */
        VI_MEASURE_VCHARGE,        /**< Measure charge voltage. */
        VI_MEASURE_GROUND,         /**< Measure ground. */
        VI_MEASURE_THEVENIN,       /**< Measure thevenin equivalent voltage. */
        VI_MEASURE_MS_DIFF,        /**< Measure HS-LS difference. */
        VI_MEASURE_UNDEFINED       ///< no definition
    };



    //--------------------------------------------------------------------------
    // Temporary for now
    typedef EnumS<VI_CONFIG_MODE>                VIConfigModeS;
    typedef EnumM<VI_CONFIG_MODE>                VIConfigModeM;
    typedef EnumS1D<VI_CONFIG_MODE>              VIConfigModeS1D;
    typedef EnumM1D<VI_CONFIG_MODE>              VIConfigModeM1D;

    typedef EnumS<VI_CONNECT_MODE>          VIConnectModeS;
    typedef EnumM<VI_CONNECT_MODE>          VIConnectModeM;
    typedef EnumM1D<VI_CONNECT_MODE>        VIConnectModeM1D;

    typedef EnumS<VI_GATE_STATE>            VIGateStateS;
    typedef EnumM<VI_GATE_STATE>            VIGateStateM;
    typedef EnumM1D<VI_GATE_STATE>          VIGateStateM1D;

    typedef EnumS<VI_MEASURE_TYPE>          VIMeasureTypeS;
    typedef EnumM<VI_MEASURE_TYPE>          VIMeasureTypeM;
    typedef EnumM1D<VI_MEASURE_TYPE>        VIMeasureTypeM1D;

  /** @} */
  /** @} */

/** @brief Class for the @ref VIAPIs "VI.*" API calls */
class UTL_VI
{
/** @addtogroup VIAPIs
@{
@ingroup Drivers
*/
public:
    /** @defgroup VI_Initialize Initialization
    @{
    @ingroup VIAPIs
    */
    /**
    @par Description
    InitializeMeasure() - Initializes the measure setup for the specified pins.
    */
    static  void    Initialize          (const PinML& Pins);
    /** @} */

    // Config
    /** @defgroup VI_Config Configuration
    @ingroup VIAPIs
    @{ */
    /**

    @par Description
    This statement configures the selected pins to support the specified
    */
    static  void        SetConfig       (const PinML& Pins, const FloatM& MaxV, const FloatM& MaxI, const VIConfigModeM& Config);
    /** @copydoc SetConfig() */
    static  void        SetConfig       (const PinML& Pins, const FloatM& MaxV, const FloatM& MaxI);

    /** @} */
    /** @} */

    UTL_VI( void );
    ~UTL_VI();

	

};//class UTL_VI

}   // namespace UTL

